﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Mogre;
using MogreNewt;

namespace TppGame
{
    public class Described : SelectableObject
    {
        public DescribedProfile Profile;

        Entity Entity;
        SceneNode Node;
        Body Body;

        public Described(DescribedProfile profile)
        {
            Profile = profile.Clone();
            
            Entity = Engine.Singleton.SceneManager.CreateEntity(Profile.MeshName);
            Node = Engine.Singleton.SceneManager.RootSceneNode.CreateChildSceneNode();
            Node.AttachObject(Entity);
        
            Vector3 scaledSize = Entity.BoundingBox.Size * Profile.BodyScaleFactor;

            ConvexCollision collision = new MogreNewt.CollisionPrimitives.Box(
                Engine.Singleton.NewtonWorld,                                
                scaledSize,
                Quaternion.IDENTITY,
                Engine.Singleton.GetUniqueBodyId());


            Vector3 inertia, offset;
            collision.CalculateInertialMatrix(out inertia, out offset);
            
            Body = new Body(Engine.Singleton.NewtonWorld, collision, true);
            Body.AttachNode(Node);
            Body.SetMassMatrix(Profile.Mass, Profile.Mass * inertia);

            Body.UserData = this;
            Body.MaterialGroupID = Engine.Singleton.MaterialManager.DescribedMaterialID;

            collision.Dispose();
        }

        public override Vector3 Position
        {
            get
            {
                return Body.Position;
            }
            set
            {
                Body.SetPositionOrientation(value, Orientation);
            }
        }

        public override void Update()
        {                                    
        }

        public override Quaternion Orientation
        {
            get
            {
                return Body.Orientation;
            }
            set
            {
                Body.SetPositionOrientation(Body.Position, value);
            }
        }

        public override string DisplayName
        {
            get
            {
                return Profile.DisplayName;
            }
            set
            {
                Profile.DisplayName = value;
            }
        }

        public override string Description
        {
            get
            {
                return Profile.Description;
            }
            set
            {
                Profile.Description = value;
            }
        }

        public override Vector3 DisplayNameOffset
        {
            get
            {
                return Profile.DisplayNameOffset;
            }
            set
            {
                Profile.DisplayNameOffset = value;
            }
        }

        public bool IsPickable
        {
            get { return Profile.IsPickable; }
        }

        public override void Destroy()
        {
            Node.DetachAllObjects();
            Engine.Singleton.SceneManager.DestroySceneNode(Node);
            Engine.Singleton.SceneManager.DestroyEntity(Entity);
            Body.Dispose();
            Body = null;

            base.Destroy();
        }
    }
}
