﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Mogre;

namespace TppGame
{
    class HumanController
    {
        public enum HumanControllerState
        {
            FREE,
            TALK,
            INVENTORY
        }

        public enum HumanTalkState
        {
            CHOOSING_REPLY,
            REPLYING,
            PAUSE,
            LISTENING
        }

        public Character Character;
        SelectableObject FocusObject;
        TextLabel3D TargetLabel;
        
        SimpleQuad TalkBox;
        List<TextLabel> TalkLabels;

        HumanControllerState State;
        HumanTalkState TalkState;

        List<TalkReply> ValidReplies;
        int SelectedReply = 0;

        float TextRemainingTime = 0.0f;        
        int TextIndex;
        TalkReply CurrentReply;
        TalkNode CurrentNode;

        HUDInventory HUDInventory;

        public HumanController()
        {            
            TargetLabel = Engine.Singleton.Labeler.NewTextLabel3D("Primitive", 0.04f, new ColourValue(0.7f, 0.4f, 0), new ColourValue(1, 1.0f, 0.6f), 0);

            TalkBox = Engine.Singleton.Labeler.NewSimpleQuad("QuadMaterial", 0.05f, 0.85f, 0.9f, 0.1f, new ColourValue(1, 1, 1), 1);
            TalkBox.IsVisible = false;
            
            TalkLabels = new List<TextLabel>();
            for (int i = 0; i < 8; i++)
            {
                TalkLabels.Add(Engine.Singleton.Labeler.NewTextLabel("Primitive", 0.04f, new ColourValue(0.7f, 0.4f, 0), new ColourValue(1, 1.0f, 0.6f), 2));
                TalkLabels[i].SetPosition(0.07f, 0.63f + i * 0.037f);
            }
            ValidReplies = new List<TalkReply>();

            HUDInventory = new HUDInventory();
        }
        

        public void HideTalkOverlay()
        {
            foreach (TextLabel lab in TalkLabels) lab.IsVisible = false;
            TalkBox.IsVisible = false;                  
        }
            

        public void ShowReplies()
        {
            TalkBox.IsVisible = true;
            foreach (TextLabel lab in TalkLabels) lab.IsVisible = true;
            TalkBox.SetDimensions(0.05f, 0.6f, 0.9f, 0.35f);
            TalkLabels.Last().SetPosition(0.07f, TalkLabels.Last().TextArea.Top);
        }

        public void ShowTalkBox()
        {
            TalkBox.IsVisible = true;
            foreach (TextLabel lab in TalkLabels) lab.IsVisible = false;
            TalkLabels.Last().IsVisible = true;            
            TalkBox.SetDimensions(0.05f, 0.85f, 0.9f, 0.1f);
        }

        public void BeginTextDisplay(TalkText text)
        {
            TalkLabels.Last().Caption = text.Text;
            TalkLabels.Last().SetPosition(0.5f - TalkLabels.Last().GetTextWidth() * 0.5f, 0.889f);
            TextRemainingTime = text.Duration;
        }

        public void SwitchState(HumanControllerState newState)
        {
            if (State == HumanControllerState.FREE)
            {
                if (newState == HumanControllerState.TALK)
                    SwitchTalkState(HumanTalkState.LISTENING);
                if (newState == HumanControllerState.INVENTORY)
                    HUDInventory.IsVisible = true;
            }
            else if (State == HumanControllerState.TALK)
            {
                if (newState == HumanControllerState.FREE)
                    HideTalkOverlay();
            }
            else if (State == HumanControllerState.INVENTORY)
                if (newState == HumanControllerState.FREE)
                    HUDInventory.IsVisible = false;
                

            State = newState;
        }

        public void SwitchTalkState(HumanTalkState newState)
        {
            if (newState == HumanTalkState.CHOOSING_REPLY)
            {
                ShowReplies();
            }
            else if (newState == HumanTalkState.PAUSE)
            {
                HideTalkOverlay();
                TextRemainingTime = 0.5f;
            }
            else if (newState == HumanTalkState.LISTENING || newState == HumanTalkState.REPLYING)
            {
                TextIndex = 0;
                if (newState == HumanTalkState.LISTENING)
                {
                    TalkLabels.Last().SetColor(new ColourValue(0.7f, 0.4f, 0), new ColourValue(1, 1.0f, 0.6f));
                    BeginTextDisplay(CurrentNode.Text[0]);
                }
                else
                {
                    TalkLabels.Last().SetColor(new ColourValue(0.4f, 0.4f, 0.4f), new ColourValue(1.0f, 1.0f, 1.0f));
                    BeginTextDisplay(CurrentReply.Text[0]);
                }

                ShowTalkBox();
            }
            TalkState = newState;
        }        

        public void UpdateRepliesColours()
        {
            for (int i = 0; i < ValidReplies.Count; i++)
            {
                if (i == SelectedReply)
                    TalkLabels[i].SetColor(new ColourValue(1, 1, 1), new ColourValue(1, 1, 1));
                else
                    TalkLabels[i].SetColor(new ColourValue(0.7f, 0.4f, 0), new ColourValue(1, 1.0f, 0.6f));
            }
        }

        public void SetRepliesText(TalkNode node)
        {
            SelectedReply = 0;
            ValidReplies.Clear();
            foreach (TextLabel label in TalkLabels)
                label.Caption = "";

            foreach (TalkReply reply in CurrentNode.Replies)
            {
                if (reply.IsConditionFulfilled())
                {
                    if (ValidReplies.Count > TalkLabels.Count)                    
                        throw new Exception("Too many valid replies in conversation");                    
                    TalkLabels[ValidReplies.Count].Caption = reply.Text[0].Text;
                    ValidReplies.Add(reply);                    
                }
            }
        }

        public void HandleConversation()
        {
            TextRemainingTime -= Engine.FixedTimeStep;

            if (TalkState == HumanTalkState.LISTENING)
            {
                if (TextRemainingTime <= 0)
                {
                    TextIndex++;
                    if (TextIndex == CurrentNode.Text.Count)
                    {
                        if (CurrentNode.Replies.Count == 0)
                        {
                            SwitchState(HumanControllerState.FREE);
                        }
                        else
                        {
                            SetRepliesText(CurrentNode);
                            SwitchTalkState(HumanTalkState.CHOOSING_REPLY);
                        }
                    }
                    else
                    {
                        BeginTextDisplay(CurrentNode.Text[TextIndex]);
                    }
                }
            }
            else if (TalkState == HumanTalkState.REPLYING)
            {
                if (TextRemainingTime <= 0)
                {
                    TextIndex++;
                    if (TextIndex == CurrentReply.Text.Count)
                    {
                        if (CurrentReply.IsEnding)
                        {
                            SwitchState(HumanControllerState.FREE);
                        }
                        else
                        {
                            CurrentNode = CurrentReply.Reaction.PickNode();                            
                            SwitchTalkState(HumanTalkState.PAUSE);
                        }
                    }
                    else
                    {
                        BeginTextDisplay(CurrentReply.Text[TextIndex]);
                    }
                }
            }
            else if (TalkState == HumanTalkState.PAUSE)
            {
                if (TextRemainingTime <= 0)
                {
                    SwitchTalkState(HumanTalkState.LISTENING);
                }
            }
            else if (TalkState == HumanTalkState.CHOOSING_REPLY)
            {
                if (Engine.Singleton.IsKeyTyped(MOIS.KeyCode.KC_DOWN))
                {
                    SelectedReply++;
                    if (SelectedReply == ValidReplies.Count)
                        SelectedReply = 0;
                }
                if (Engine.Singleton.IsKeyTyped(MOIS.KeyCode.KC_UP))
                {
                    SelectedReply--;
                    if (SelectedReply == -1)
                        SelectedReply = ValidReplies.Count - 1;
                }
                UpdateRepliesColours();

                if (Engine.Singleton.Keyboard.IsKeyDown(MOIS.KeyCode.KC_SPACE))
                {                    
                    CurrentReply = ValidReplies[SelectedReply];
                    SwitchTalkState(HumanTalkState.REPLYING);                    
                }
            }
        }

        private void HandleInventory()
        {
            if (Engine.Singleton.IsKeyTyped(MOIS.KeyCode.KC_DOWN))
                HUDInventory.SelectIndex++;
            if (Engine.Singleton.IsKeyTyped(MOIS.KeyCode.KC_UP))
                HUDInventory.SelectIndex--;

            if (Engine.Singleton.IsKeyTyped(MOIS.KeyCode.KC_LMENU))
            {
                if (HUDInventory.SelectIndex != -1)
                {
                    if (Character.DropItem(HUDInventory.SelectIndex))
                    {                        
                        HUDInventory.SelectIndex = HUDInventory.SelectIndex;
                        HUDInventory.UpdateView();
                    }
                }
            }

            if (Engine.Singleton.IsKeyTyped(MOIS.KeyCode.KC_RCONTROL))
            {
                if (HUDInventory.SelectIndex != -1)
                {
                    if (Character.Inventory[HUDInventory.SelectIndex] is ItemSword)
                    {
                        if (Character.Sword != Character.Inventory[HUDInventory.SelectIndex])                        
                            Character.Sword = Character.Inventory[HUDInventory.SelectIndex] as ItemSword;                                                    
                        else
                            Character.Sword = null;
                        HUDInventory.UpdateItem(HUDInventory.SelectIndex);
                    }
                }
            }

            if (Engine.Singleton.IsKeyTyped(MOIS.KeyCode.KC_TAB))
                SwitchState(HumanControllerState.FREE);
        }

        public void Update()
        {
            if (Character != null)
            {
                if (State == HumanControllerState.FREE)
                    HandleMovement();
                else if (State == HumanControllerState.TALK)
                    HandleConversation(); 
                else if (State == HumanControllerState.INVENTORY)
                    HandleInventory();
            }
        }

        private void HandleMovement()
        {
            Quaternion rotation = new Quaternion();
            rotation.FromAngleAxis(new Degree(2), Vector3.UNIT_Y);

            if (Engine.Singleton.Keyboard.IsKeyDown(MOIS.KeyCode.KC_RCONTROL))
            {
                if (FocusObject != null)
                {
                    if (FocusObject is Described)
                    {
                        if ((FocusObject as Described).IsPickable)
                            Character.TryPick(FocusObject as Described);
                    }
                }
            }

            if (Engine.Singleton.Keyboard.IsKeyDown(MOIS.KeyCode.KC_SPACE))
            {
                if (FocusObject != null)
                {
                    if (Character.TalkPerm && FocusObject.TalkRoot != null)
                    {
                        CurrentNode = FocusObject.TalkRoot.PickNode();
                        SwitchState(HumanControllerState.TALK);
                    }
                }
            }

            if (Engine.Singleton.IsKeyTyped(MOIS.KeyCode.KC_TAB)
                && Character.InventoryPerm) 
                SwitchState(HumanControllerState.INVENTORY);

            
            if (Engine.Singleton.Keyboard.IsKeyDown(MOIS.KeyCode.KC_LEFT))
                Character.TurnDelta = 2;
            if (Engine.Singleton.Keyboard.IsKeyDown(MOIS.KeyCode.KC_RIGHT))
                Character.TurnDelta = -2;

            if (Engine.Singleton.Keyboard.IsKeyDown(MOIS.KeyCode.KC_UP))
                Character.MoveOrder = true;
            else
                Character.MoveOrder = false;

            if (Character.Contacts.Count > 0)
            {
                SelectableObject contact = Character.Contacts[0] as SelectableObject;
                FocusObject = contact;
                TargetLabel.Caption = contact.DisplayName;
                TargetLabel.Position3D = contact.Position + contact.DisplayNameOffset;
                TargetLabel.IsVisible = true;
            }
            else
            {
                FocusObject = null;
                TargetLabel.IsVisible = false;
            }
        }
    }
}
