﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using Mogre;

namespace TppGame
{
    class Program
    {
        static void Main(string[] args)
        {
            Engine.Singleton.Initialise();
            
            Engine.Singleton.CurrentLevel = new Level();
            Engine.Singleton.CurrentLevel.SetGraphicsMesh("World.mesh");
            Engine.Singleton.CurrentLevel.SetCollisionMesh("World.mesh");
            Engine.Singleton.SceneManager.AmbientLight = new ColourValue(0,0,0);

            TriggerVolume triggerVolume = new TriggerVolume();
            triggerVolume.BeginShapeBuild();
            triggerVolume.AddBoxPart(Vector3.ZERO, Quaternion.IDENTITY, new Vector3(2, 2, 2));
            triggerVolume.EndShapeBuild();
            Engine.Singleton.ObjectManager.Add(triggerVolume);
            triggerVolume.Position = new Vector3(4, 1, 0);
            triggerVolume.OnCharacterEntered += new TriggerVolume.CharacterEnteredHandler(triggerVolume_OnCharacterEntered);

            CharacterProfile profile = new CharacterProfile();
            profile.BodyMass = 70;
            profile.BodyScaleFactor = new Vector3(1.5f, 1, 1.5f);
            profile.HeadOffset = new Vector3(0, 0.8f, 0);
            profile.MeshName = "Man.mesh";
            profile.WalkSpeed = 1.85f;            

            DescribedProfile vaseProfile = new DescribedProfile();            
            vaseProfile.BodyScaleFactor = new Vector3(0.7f, 1, 0.7f);
            vaseProfile.MeshName = "Vase.mesh";
            vaseProfile.DisplayName = "Vase";
            vaseProfile.Description = "Vase made of clay";
            vaseProfile.DisplayNameOffset = new Vector3(0.0f, 0.7f, 0.0f);

            ItemSword swordProfile = new ItemSword();
            swordProfile.BodyScaleFactor = new Vector3(1.0f, 1.0f, 1.0f);
            swordProfile.MeshName = "Sword.mesh";
            swordProfile.DisplayName = "Sword";
            swordProfile.Description = 
@"A sword is a long, edged piece of 
forged metal, used in many civilizations 
throughout the world, primarily as a 
cutting or thrusting weapon and 
occasionally as a clubbing or 
bludgeoning weapon. The word sword 
comes from the Old English sweord";
            swordProfile.InventoryPictureMaterial = "SpriteSword";
            swordProfile.DisplayNameOffset = new Vector3(0.0f, 0.2f, 0.0f);
            swordProfile.Mass = 50;
            swordProfile.IsPickable = true;
            swordProfile.Damage = 1;
            swordProfile.HandleOffset = new Vector3(0, 0.3f, 0);
     
            Described sword = new Described(swordProfile);
            sword.Position = new Vector3(5.5f, 1.5f, 0.0f);
            Engine.Singleton.ObjectManager.Add(sword);           

            Character player = new Character(profile);
            player.Position = new Vector3(0, 0, -5);
            Engine.Singleton.ObjectManager.Add(player);

            Character npc = new Character(profile);
            npc.Position = new Vector3(7, 1, -18);
            npc.TalkRoot = Conversations.ConversationRoot;
            npc.DisplayNameOffset = new Vector3(0, 1, 0);
            npc.Profile.DisplayName = "The Guy";
            npc.TurnTo(Vector3.ZERO);
            Engine.Singleton.ObjectManager.Add(npc);

            Engine.Singleton.GameCamera.Character = player;
            Engine.Singleton.GameCamera.Distance = 4;
            Engine.Singleton.GameCamera.Angle = new Degree(20);

            CreateCrystalLight(new Vector3(11.1f, 1.18f, -4.8f));
            CreateCrystalLight(new Vector3(13.1f, -0.44f, -12.68f));
            CreateCrystalLight(new Vector3(9.34f, 1.57f, -20.61f));
            CreateCrystalLight(new Vector3(22.85f, -0.44f, -10.15f));
           

            Light light = Engine.Singleton.SceneManager.CreateLight();
            light.Type = Light.LightTypes.LT_POINT;
            //light.Direction = new Vector3(1, -3, 1).NormalisedCopy;
            light.Position = new Vector3(0, 0, 0);
            light.DiffuseColour = new ColourValue(1f, 0.8f, 0.55f);
            light.SetAttenuation(32, 1.0f, 0.14f, 0.07f);           

            //light.CastShadows = false;

            SceneNode node = Engine.Singleton.SceneManager.RootSceneNode.CreateChildSceneNode(new Vector3(0, -0.95f, 0));
            ParticleSystem ps = Engine.Singleton.SceneManager.CreateParticleSystem("Ognisko1a", "Particles/Fire");  
            node.AttachObject(ps);
            
            ps = Engine.Singleton.SceneManager.CreateParticleSystem("Ognisko1b", "Particles/Fire2");
            node.AttachObject(ps);         
           
            Mogre.MaterialManager.Singleton.SetDefaultTextureFiltering(TextureFilterOptions.TFO_ANISOTROPIC);
            Mogre.MaterialManager.Singleton.DefaultAnisotropy = 4;

            Engine.Singleton.SceneManager.ShadowTechnique = ShadowTechnique.SHADOWTYPE_STENCIL_ADDITIVE;            


            Engine.Singleton.HumanController.Character = player;

            int frame=15;
            Vector3 targetAtten = new Vector3();
            Vector3 targetPos = new Vector3();
            while (true)
            {                
                if (frame <= 0)
                {
                    targetAtten = new Vector3(1.0f, Mogre.Math.RangeRandom(0.01f, 0.25f), Mogre.Math.RangeRandom(0.02f, 0.15f));
                    targetPos = new Vector3(light.Position.x, Mogre.Math.RangeRandom(0, 0.4f), light.Position.z);      
                    frame = (int)(Mogre.Math.UnitRandom() * 15.0f);
                }
                frame--;
                light.SetAttenuation(32, 1.0f, light.AttenuationLinear + (targetAtten.y - light.AttenuationLinear) * 0.1f,
                         light.AttenuationQuadric + (targetAtten.z - light.AttenuationQuadric) * 0.1f);
                light.Position += (targetPos - light.Position) * 0.1f; 
                Engine.Singleton.Update();
              
                if (Engine.Singleton.Keyboard.IsKeyDown(MOIS.KeyCode.KC_ESCAPE))
                    break;

                if (Engine.Singleton.Keyboard.IsKeyDown(MOIS.KeyCode.KC_F3))
                    Engine.Singleton.NewtonDebugger.ShowDebugInformation();
                else
                    Engine.Singleton.NewtonDebugger.HideDebugInformation();
            }

            Engine.Singleton.Root.Dispose();
             
        }

        static void CreateCrystalLight(Vector3 pos)
        {
            Light crystalLight = Engine.Singleton.SceneManager.CreateLight();
            crystalLight.Type = Light.LightTypes.LT_POINT;
            crystalLight.Position = pos;
            crystalLight.DiffuseColour = new ColourValue(0.2f, 0.8f, 1.0f);
            crystalLight.SetAttenuation(32, 1.0f, 0.14f, 0.13f);
           // crystalLight.CastShadows = false;
        }

        static void triggerVolume_OnCharacterEntered(TriggerVolume sender, Character character)
        {
            Console.WriteLine("Bohater wszedł na teren wyzwalacza");
        }
    }
}
